import os
import sys
from textwrap import dedent

from mock import patch
from moban.main import main
from nose.tools import eq_
from fs.opener.parse import parse_fs_url
from moban.externals import file_system


def verify_content(file_name, expected):
    with open(file_name, "r") as f:
        content = f.read()
        eq_(content, expected)


def verify_content_with_fs(file_name, expected):
    content = file_system.read_unicode(file_name)
    eq_(content, expected)


def run_moban(args, folder, criterias):
    with patch.object(sys, "argv", args):
        main()
        for output, expected in criterias:
            verify_content(output, expected)
            os.unlink(output)


def run_moban_with_fs(args, folder, criterias):
    with patch.object(sys, "argv", args):
        main()

        for output, expected in criterias:
            verify_content_with_fs(output, expected)
    result = parse_fs_url(output)
    os.unlink(result.resource)  # delete the zip file


class Docs(object):
    def setUp(self):
        self.current = os.getcwd()
        self.base_folder = "docs"

    def tearDown(self):
        if os.path.exists(".moban.hashes"):
            os.unlink(".moban.hashes")
        os.chdir(self.current)

    def run_moban(self, moban_cli, working_directory, assertions):
        os.chdir(os.path.join(self.base_folder, working_directory))
        run_moban(moban_cli, None, assertions)

    def run_moban_with_fs(self, moban_cli, working_directory, assertions):
        os.chdir(os.path.join(self.base_folder, working_directory))
        run_moban_with_fs(moban_cli, None, assertions)


def custom_dedent(long_texts):
    refined = dedent(long_texts)
    if refined.startswith("\n"):
        refined = refined[1:]
    return refined



from pybars import Compiler
from lml.plugin import PluginInfo

from moban_handlebars.constants import HELPER_EXTENSION, PARTIALS_EXTENSION


class Helper(PluginInfo):
    """
    @Helper('tag_used_inside_handlebar')
    def the_actual_implementation(this, options, items):
        ...
    """
    def __init__(self, helper):
        super(Helper, self).__init__(HELPER_EXTENSION)
        self.helper = helper

    def tags(self):
        yield self.helper


def register_partial(identifier, partial):
    plugin = PluginInfo(PARTIALS_EXTENSION, tags=[identifier])
    partial = Compiler().compile(partial)
    plugin({identifier: partial})



from typing import Dict

from pybars import Compiler
from lml.plugin import PluginManager
from moban.externals import file_system

from moban_handlebars.constants import HELPER_EXTENSION, PARTIALS_EXTENSION


class HandlebarsHelperManager(PluginManager):
    def __init__(self):
        super(HandlebarsHelperManager, self).__init__(HELPER_EXTENSION)

    def get_all(self):
        for name in self.registry.keys():
            the_helper = self.load_me_now(name)
            yield (name, the_helper)


class HandlebarsPartialManager(PluginManager):
    def __init__(self):
        super(HandlebarsPartialManager, self).__init__(PARTIALS_EXTENSION)

    def get_all(self):
        for name in self.registry.keys():
            partial = self.load_me_now(name)
            yield (name, partial[name])


HELPERS = HandlebarsHelperManager()
PARTIALS = HandlebarsPartialManager()


class EngineHandlebars(object):
    ACTION_IN_PRESENT_CONTINUOUS_TENSE = "Handlebars-ing"
    ACTION_IN_PAST_TENSE = "Handlebarsed"

    def __init__(self, template_fs, options: Dict = None):
        """
        template_fs is a multfs instance and gives you the power to load
        a template from equiped template directories.

        :param fs.multifs.MultiFS template_fs: a MultiFS/FS instance
        :param options: a dictionary of potential parameters.
                        not used yet.
        """
        self.template_fs = template_fs

    def get_template(self, template_file: str):
        template_file = file_system.to_unicode(template_file)
        content = self.template_fs.readtext(template_file)
        hbr_template = Compiler().compile(content)
        return hbr_template

    def get_template_from_string(self, string: str):
        string = file_system.to_unicode(string)
        return Compiler().compile(string)

    def apply_template(self, template, data: Dict, _):
        helpers = HELPERS.get_all()
        helpers = dict(list(helpers))

        partials = PARTIALS.get_all()
        partials = dict(list(partials))
        rendered_content = "".join(
            template(data, helpers=helpers, partials=partials)
        )
        rendered_content = rendered_content
        return rendered_content



from moban_handlebars.api import Helper, register_partial

register_partial("header", "<h1>People</h1>")


@Helper("list")
def _list(this, options, items):
    result = [u"<ul>"]
    for thing in items:
        result.append(u"<li>")
        result.extend(options["fn"](thing))
        result.append(u"</li>")
    result.append(u"</ul>")
    return result



import fs.path

from .utils import Docs


class TestRegression(Docs):
    def setUp(self):
        super(TestRegression, self).setUp()
        self.base_folder = fs.path.join("tests", "regression_tests")

    def test_helper_and_partial(self):
        expected = (
            "<h1>People</h1>"
            + "<ul><li>Bill 100</li><li>Bob 90</li><li>Mark 25</li></ul>"
        )

        folder = "level-11-helpers-and-partials"
        self.run_moban(
            [
                "moban",
                "-pd",
                "helper_and_partial",
                "-c",
                "data.json",
                "--template-type",
                "hbs",
                '{{>header}}{{#list people}}{{name}} {{age}}{{/list}}',
            ],
            folder,
            [("moban.output", expected)],
        )



# flake8: noqa
import moban.constants as constants
from lml.plugin import PluginInfo, PluginInfoChain

from moban_handlebars._version import __author__, __version__

PluginInfoChain(__name__).add_a_plugin_instance(
    PluginInfo(
        constants.TEMPLATE_ENGINE_EXTENSION,
        "%s.engine.EngineHandlebars" % __name__,
        tags=["handlebars", "hbs"],
    )
)



import os

from moban.core import ENGINES, plugins
from nose.tools import eq_
from moban.externals import file_system

from moban_handlebars.engine import EngineHandlebars

plugins.make_sure_all_pkg_are_loaded()


def test_handlebars_template_not_found():
    path = os.path.join("tests", "fixtures", "handlebars_tests")
    engine = EngineHandlebars(file_system.get_multi_fs([path]))
    template = engine.get_template("file_tests.template")
    data = dict(test="here")
    result = engine.apply_template(template, data, None)
    expected = "here"
    eq_(expected, result)


def test_handlebars_template_type():
    engine = ENGINES.get_engine("hbs", [], "")
    assert engine.engine.__class__.__name__ == "EngineHandlebars"


def test_handlebars_file_tests():
    output = "test.txt"
    path = os.path.join("tests", "fixtures", "handlebars_tests")
    engine = ENGINES.get_engine("hbs", [path], path)
    engine.render_to_file("file_tests.template", "file_tests.json", output)
    with open(output, "r") as output_file:
        content = output_file.read()
        eq_(content, "here")
    os.unlink(output)


def test_handlebars_string_template():
    template_string = "{{test}}"
    output = "test.txt"
    path = os.path.join("tests", "fixtures", "handlebars_tests")
    engine = ENGINES.get_engine("hbs", [path], path)
    engine.render_string_to_file(template_string, "file_tests.json", output)
    with open(output, "r") as output_file:
        content = output_file.read()
        eq_(content, "here")
    os.unlink(output)



#!/usr/bin/env python3

"""
Template by pypi-mobans
"""

import os
import sys
import codecs
import locale
import platform
from shutil import rmtree

from setuptools import Command, setup, find_packages

PY2 = sys.version_info[0] == 2
PY26 = PY2 and sys.version_info[1] < 7
PY33 = sys.version_info < (3, 4)

# Work around mbcs bug in distutils.
# http://bugs.python.org/issue10945
# This work around is only if a project supports Python < 3.4

# Work around for locale not being set
try:
    lc = locale.getlocale()
    pf = platform.system()
    if pf != "Windows" and lc == (None, None):
        locale.setlocale(locale.LC_ALL, "C.UTF-8")
except (ValueError, UnicodeError, locale.Error):
    locale.setlocale(locale.LC_ALL, "en_US.UTF-8")

NAME = "moban-handlebars"
AUTHOR = "Charlie Liu, C.W. et al"
VERSION = "0.0.5"
EMAIL = "wangc_2011@hotmail.com"
LICENSE = "MIT"
DESCRIPTION = (
    "Provide handlebars templating capability to moban."
)
URL = "https://github.com/moremoban/moban-handlebars"
DOWNLOAD_URL = "%s/archive/0.0.5.tar.gz" % URL
FILES = ["README.rst", 'CONTRIBUTORS.rst', "CHANGELOG.rst"]
KEYWORDS = [
    "python",
]

CLASSIFIERS = [
    "Topic :: Software Development :: Libraries",
    "Programming Language :: Python",
    "Intended Audience :: Developers",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",

]


INSTALL_REQUIRES = [
    "pybars3",
    "lml>=0.0.9",
]
SETUP_COMMANDS = {}

PACKAGES = find_packages(exclude=["ez_setup", "examples", "tests", "tests.*"])
EXTRAS_REQUIRE = {
}
# You do not need to read beyond this line
PUBLISH_COMMAND = "{0} setup.py sdist bdist_wheel upload -r pypi".format(sys.executable)
GS_COMMAND = ("gs moban-handlebars v0.0.5 " +
              "Find 0.0.5 in changelog for more details")
NO_GS_MESSAGE = ("Automatic github release is disabled. " +
                 "Please install gease to enable it.")
UPLOAD_FAILED_MSG = (
    'Upload failed. please run "%s" yourself.' % PUBLISH_COMMAND)
HERE = os.path.abspath(os.path.dirname(__file__))


class PublishCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package on github and pypi"
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds...")
            rmtree(os.path.join(HERE, "dist"))
            rmtree(os.path.join(HERE, "build"))
            rmtree(os.path.join(HERE, "moban_handlebars.egg-info"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution...")
        run_status = True
        if has_gease():
            run_status = os.system(GS_COMMAND) == 0
        else:
            self.status(NO_GS_MESSAGE)
        if run_status:
            if os.system(PUBLISH_COMMAND) != 0:
                self.status(UPLOAD_FAILED_MSG)

        sys.exit()


SETUP_COMMANDS.update({
    "publish": PublishCommand
})


def has_gease():
    """
    test if github release command is installed

    visit http://github.com/moremoban/gease for more info
    """
    try:
        import gease  # noqa
        return True
    except ImportError:
        return False


def read_files(*files):
    """Read files into setup"""
    text = ""
    for single_file in files:
        content = read(single_file)
        text = text + content + "\n"
    return text


def read(afile):
    """Read a file into setup"""
    the_relative_file = os.path.join(HERE, afile)
    with codecs.open(the_relative_file, "r", "utf-8") as opened_file:
        content = filter_out_test_code(opened_file)
        content = "".join(list(content))
        return content


def filter_out_test_code(file_handle):
    found_test_code = False
    for line in file_handle.readlines():
        if line.startswith(".. testcode:"):
            found_test_code = True
            continue
        if found_test_code is True:
            if line.startswith("  "):
                continue
            else:
                empty_line = line.strip()
                if len(empty_line) == 0:
                    continue
                else:
                    found_test_code = False
                    yield line
        else:
            for keyword in ["|version|", "|today|"]:
                if keyword in line:
                    break
            else:
                yield line


if __name__ == "__main__":
    setup(
        test_suite="tests",
        name=NAME,
        author=AUTHOR,
        version=VERSION,
        author_email=EMAIL,
        description=DESCRIPTION,
        url=URL,
        download_url=DOWNLOAD_URL,
        long_description=read_files(*FILES),
        license=LICENSE,
        keywords=KEYWORDS,
        extras_require=EXTRAS_REQUIRE,
        tests_require=["nose"],
        install_requires=INSTALL_REQUIRES,
        packages=PACKAGES,
        include_package_data=True,
        zip_safe=False,
        classifiers=CLASSIFIERS,
        cmdclass=SETUP_COMMANDS
    )
